//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Log.cpp
 *  \brief Implementacia bazovej triedy logovacich vypisov hraca
 *  \author t0n0 Weissensteiner & Mato Spal
 *  \date 26.2.2002 22:43:41
 *  \version 1.0
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLog::CLog(
	const string &sFileName, 
	const LOG_MASK eMsgMask /* = LOG_MASK_EMPTY */,
	const string &sFileMode /* = LOG_ACCESS_MODE_W */
)
:	m_eMsgMask(LOG_MASK_EMPTY)
{
#ifndef _NO_LOG_FILES_

	if(cLogManager.Open(sFileName, sFileMode) == KPDL_SUCCESS)
	{
		// ak sa podarilo otvorit subor, zapametame si jeho meno
		m_sFileName = sFileName;
		m_eMsgMask = eMsgMask;
	}

#endif // _NO_LOG_FILES_
}

CLog::~CLog()
{
#ifndef _NO_LOG_FILES_

	// zatvorenie suboru
	if(!m_sFileName.empty())
	{
		cLogManager.Close(m_sFileName);
		m_sFileName.erase();
		m_eMsgMask = LOG_MASK_EMPTY;
	}

#endif // _NO_LOG_FILES_
}

KPDL_RESULT CLog::Write(
	const bool bTimeStamp,
	const char *sFormat,
	...
)
{
#ifndef _NO_LOG_FILES_
	
	if(m_sFileName.empty())
		return KPDL_UNSUCCESS;

	va_list arglist;

	va_start(arglist, sFormat);
	vsprintf(m_sBuffer, sFormat, arglist);
	va_end(arglist);

	return cLogManager.Write(
		m_sFileName, 
		m_eMsgMask, 
		bTimeStamp, 
		m_sBuffer
	);

#else // _NO_LOG_FILES_

	return KPDL_SUCCESS;

#endif // _NO_LOG_FILES_
}
