// LogManager.cpp: implementation of the CLogManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLogManager::CLogManager(
	const LOG_MASK eViewMask /* = LOG_MASK_SYSTEM */,
	CLogView *pLogView /* = NULL */
)
:	m_eViewMask(eViewMask),
	pLogView(pLogView)
{
	m_ulRefTimeStamp = GetTickCount();
}

CLogManager::~CLogManager()
{
	TFileMapIterator pFileIter = m_cFileMap.begin();
	TFileMapIterator pFileLast = m_cFileMap.end();
	FILE *hFileHnd;

	while(pFileIter != pFileLast)
	{
		hFileHnd = (*pFileIter).second.hFileHnd;
		if(hFileHnd != NULL)
			fclose(hFileHnd);

		pFileIter++;
	}

	m_cFileMap.clear();
}

KPDL_RESULT CLogManager::SetViewMask(
	const LOG_MASK eViewMask
)
{
	m_eViewMask = eViewMask;

	if(m_eViewMask == LOG_MASK_EMPTY)
	{
		cLogBuffer.Clear();
		if(pLogView != NULL)
			pLogView->Invalidate();
	}

	return KPDL_SUCCESS;
}

KPDL_RESULT CLogManager::Open(
	const string &sFileName, 
	const string &sFileMode
)
{
	SFileEntry tNewEntry;
	TFileMapIterator pFileIter = m_cFileMap.find(sFileName);
	TFileMapIterator pFileLast = m_cFileMap.end();

	if(pFileIter != pFileLast)
	{
		// Subor je uz otvoreny
		(*pFileIter).second.ulRefCount++;
		return KPDL_SUCCESS;
	}

	tNewEntry.ulRefCount = 1L;
	tNewEntry.hFileHnd = fopen(sFileName.c_str(), sFileMode.c_str());
	if(tNewEntry.hFileHnd != NULL)
	{
		// Pridame novy subor do tabulky
		m_cFileMap[sFileName] = tNewEntry;
		return KPDL_SUCCESS;
	}

	return KPDL_UNSUCCESS;
}

KPDL_RESULT CLogManager::Close(const string &sFileName)
{
	TFileMapIterator pFileIter = m_cFileMap.find(sFileName);
	TFileMapIterator pFileLast = m_cFileMap.end();

	if(pFileIter == pFileLast)
		return KPDL_UNSUCCESS;

	SFileEntry &tEntry = (*pFileIter).second;

	// Subor je este otvoreny
	fflush(tEntry.hFileHnd);
	if(! --tEntry.ulRefCount)
	{
		fclose(tEntry.hFileHnd);
		// just to be sure ...
		tEntry.hFileHnd = NULL;
		m_cFileMap.erase(sFileName);
	}

	return KPDL_SUCCESS;
}

KPDL_RESULT CLogManager::Write(
	const string &sFileName,
	const LOG_MASK eMsgMask,
	const bool bTimeStamp,
	const char* sMessage
)
{
	static char sTimeStamp[LOG_MAX_TIMESTAMP];
	TFileMapIterator pFileIter = m_cFileMap.find(sFileName);
	TFileMapIterator pFileLast = m_cFileMap.end();

	if(pFileIter == pFileLast)
		return KPDL_UNSUCCESS;

	if(eMsgMask & m_eViewMask)
	{
		SFileEntry &tEntry = (*pFileIter).second;

		if(bTimeStamp)
			sprintf(sTimeStamp, "%010d", GetTickCount() - m_ulRefTimeStamp);
		else
			sTimeStamp[0] = '\0';

		fprintf(tEntry.hFileHnd, "%10s: %s", sTimeStamp, sMessage);
		fputc('\n', tEntry.hFileHnd);

		// Add message to the message buffer
		cLogBuffer.AddMsg(sFileName, sTimeStamp, sMessage);

		if(pLogView != NULL)
			pLogView->Invalidate();
	}

	return KPDL_SUCCESS;
}
