// CLogView.cpp : implementation file
//

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLogView

CLogView::CLogView()
:	m_pDevice(new CDC()),
	m_pBitmap(new CBitmap()),
	m_pFont(new CFont())
{
	cLogManager.pLogView = this;
}

CLogView::~CLogView()
{
	cLogManager.pLogView = NULL;

	if(m_pFont != NULL)
	{
		delete m_pFont;
		m_pFont = NULL;
	}
	if(m_pBitmap != NULL)
	{
		delete m_pBitmap;
		m_pBitmap = NULL;
	}
	if(m_pDevice != NULL)
	{
		delete m_pDevice;
		m_pDevice = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
// BOOL CLogView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
// ---------------------------------------------------------------------------
// Description:
//		
//
// ---------------------------------------------------------------------------
// Parameters:
//		LPCTSTR lpszClassName
//		LPCTSTR lpszWindowName
//		DWORD dwStyle
//		const RECT& rect
//		CWnd* pParentWnd
//		UINT nID
//		CCreateContext* pContext
//
//////////////////////////////////////////////////////////////////////////////
BOOL CLogView::Create(
	LPCTSTR lpszClassName, 
	LPCTSTR lpszWindowName, 
	DWORD dwStyle, 
	const RECT& rect, 
	CWnd* pParentWnd, 
	UINT nID, 
	CCreateContext* pContext
) 
{	
	BOOL bRetVal = CWnd::Create(
		lpszClassName, 
		lpszWindowName, 
		dwStyle, 
		rect, 
		pParentWnd, 
		nID, 
		pContext
	);

	CClientDC cClientDC(this);
	m_pDevice->CreateCompatibleDC(&cClientDC);
	m_pBitmap->CreateCompatibleBitmap(
		&cClientDC, 
		::GetSystemMetrics(SM_CXSCREEN),
		::GetSystemMetrics(SM_CYSCREEN)
	); 
	m_pDevice->SetMapMode(MM_TEXT);
	m_pFont->CreateFont(
		LOG_FONT_SIZE, 0, 0, 0, FW_DONTCARE,
		FALSE, FALSE, FALSE, ANSI_CHARSET, 
		OUT_DEVICE_PRECIS, CLIP_DEFAULT_PRECIS, 
		DEFAULT_QUALITY, DEFAULT_PITCH, LOG_FONT_NAME
	);
	m_pDevice->SelectObject(m_pFont);
	return bRetVal;
}

//////////////////////////////////////////////////////////////////////////////
// BOOL CLogView::PreCreateWindow(CREATESTRUCT& tCS) 
// ---------------------------------------------------------------------------
// Description:
//		
//
// ---------------------------------------------------------------------------
// Parameters:
//		CREATESTRUCT& cs
//
//////////////////////////////////////////////////////////////////////////////
BOOL CLogView::PreCreateWindow(CREATESTRUCT& tCS) 
{
	if(!CWnd::PreCreateWindow(tCS))
		return FALSE;

	tCS.dwExStyle |= WS_EX_CLIENTEDGE;
	tCS.style &= ~WS_BORDER;
	tCS.lpszClass = AfxRegisterWndClass(
		CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW),
		HBRUSH(COLOR_WINDOW+1),
		NULL
	);
	
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CLogView message handlers

BEGIN_MESSAGE_MAP(CLogView, CWnd)
	//{{AFX_MSG_MAP(CLogView)
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CLogView::OnEraseBkgnd(CDC* pDevice) 
{
	return FALSE;
}

//////////////////////////////////////////////////////////////////////////////
// void CLogView::OnPaint() 
// ---------------------------------------------------------------------------
// Description:
//		
//
//////////////////////////////////////////////////////////////////////////////
void CLogView::OnPaint() 
{
	CPaintDC cDevice(this);
	CRect cRectUpd;

	cDevice.GetClipBox(&cRectUpd);

	CBitmap* pOldBitmap = m_pDevice->SelectObject(m_pBitmap);
	m_pDevice->SelectClipRgn(0);
	m_pDevice->IntersectClipRect(&cRectUpd);

	CBrush cBkgndBrush((COLORREF)::GetSysColor(COLOR_WINDOW));
	CBrush* pOldBrush = m_pDevice->SelectObject(&cBkgndBrush);
	m_pDevice->PatBlt(
		cRectUpd.left, 
		cRectUpd.top, 
		cRectUpd.Width(), 
		cRectUpd.Height(), 
		PATCOPY
	);

	cLogManager.cLogBuffer.Show(*m_pDevice, m_cViewSize.y, LOG_FONT_SIZE);

	cDevice.BitBlt(
		cRectUpd.left, 
		cRectUpd.top, 
		cRectUpd.Width(), 
		cRectUpd.Height(), 
		m_pDevice, 
		cRectUpd.left, 
		cRectUpd.top, 
		SRCCOPY
	);
	m_pDevice->SelectObject(pOldBitmap);
	m_pDevice->SelectObject(pOldBrush);
}

void CLogView::OnSize(UINT uiType, int iX, int iY) 
{
	m_cViewSize.x = iX;
	m_cViewSize.y = iY;
}
