//*****************************************************************************
// Activate.cpp: Implementation of neural activation function
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// CBinaryUnipol
//*****************************************************************************
CBinaryUnipol::CBinaryUnipol()
:	CActivate(0.0, 1.0)
{
}

double CBinaryUnipol::F(double dNet)
{
	if(dNet > 0.0)
		return dMax;
	return dMin;
}

double CBinaryUnipol::DiffF(double dO)
{
	// No influence in the learning process
	return 1.0;
}

//*****************************************************************************
// CBinaryBipol
//*****************************************************************************
CBinaryBipol::CBinaryBipol()
:	CActivate(-1.0, 1.0)
{
}
	
double CBinaryBipol::F(double dNet)
{
	if(dNet > 0.0)
		return dMax;
	return dMin;
}

double CBinaryBipol::DiffF(double dO)
{
	// No influence in the learning process
	return 1.0;
}

//*****************************************************************************
// CContinuosUnipol
//*****************************************************************************
CContinuousUnipol::CContinuousUnipol()
:	CActivate(0.0, 1.0)
{
}

double CContinuousUnipol::F(double dNet)
{
	// Because of floating point overflow
	if(dNet > EXP_PREC_LIMIT)
		return dMax;

	if(dNet < -EXP_PREC_LIMIT)
		return dMin;

	return 1.0 / (1.0 + exp(-dNet));
}

double CContinuousUnipol::DiffF(double dO)
{
	return dO * (1 - dO);
}

//*****************************************************************************
// CContinuousBipol
//*****************************************************************************
CContinuousBipol::CContinuousBipol()
:	CActivate(-1.0, 1.0)
{}

double CContinuousBipol::F(double dNet)
{
	// Because of floating point overflow
	if(dNet > EXP_PREC_LIMIT)
		return dMax;

	if(dNet < -EXP_PREC_LIMIT)
		return dMin;

	return 2.0 / (1.0 + exp(-dNet)) - 1.0;
}

double CContinuousBipol::DiffF(double dO)
{
	return 0.5 * (1 - dO * dO);
}

