//*****************************************************************************
// Forward.cpp: Implementation of the CForwardNetwork class
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CForwardNetwork::CForwardNetwork(
	const int iSizeIn,
	const int iSizeHid,
	const int iSizeOut,
	CActivate &cActivate,
	const double dMomentum, /* = 0.0 */
	const double dAlpha /* = 0.0 */
)
:	// Limits used
	m_iSizeIn(iSizeIn),
	m_iSizeHid(iSizeHid),
	m_iSizeOut(iSizeOut),
	// Configuration
	dMomentum(dMomentum),
	dAlpha(dAlpha),
	// Weights
	// Weights matrices
	m_cWeightsHid(iSizeHid, iSizeIn, -1.0, 1.0),
	m_cWeightsOut(iSizeOut, iSizeHid, -1.0, 1.0),
	m_cDeltaHid(iSizeHid, iSizeIn),
	m_cDeltaOut(iSizeOut, iSizeHid),
	// Temporal output from hidden layer
	m_cOutputHid(iSizeHid),
	m_cOutputOut(iSizeOut),
	// Activation function
	m_cActivate(cActivate)
{
	puts(" - Hidden weights: ");
	m_cWeightsHid.Print(" % 6.4f");
	puts("\n - Output weights: ");
	m_cWeightsOut.Print(" % 6.4f");
	puts("");
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

void CForwardNetwork::Process(
	CVector &cInput, 
	CVector &cOutput, 
	double dTolerance
)
{
	int iNeuron;
	int iInput;
	double dNet;
	
	// Ouptut from hidden layer
	for(iNeuron = 0; iNeuron < m_iSizeHid; iNeuron++)
	{
		dNet = 0.0;
		for(iInput = 0; iInput < m_iSizeIn; iInput++)
			dNet += cInput(iInput) * m_cWeightsHid(iNeuron, iInput);
//		m_cOutputHid(iNeuron) = F(dNet);
	}
	// Dummy neuron :)
	m_cOutputHid(m_iSizeHid - 1) = -1.0;

	// Output from network
	for(iNeuron = 0; iNeuron < m_iSizeOut; iNeuron++)
	{
		dNet = 0.0;
		for(iInput = 0; iInput < m_iSizeHid; iInput++)
			dNet += m_cOutputHid(iInput) * m_cWeightsOut(iNeuron, iInput);
//		m_cOutputOut(iNeuron) = F(dNet);
	}

	// Fit the output according to the tolerance
	for(iNeuron = 0; iNeuron < m_iSizeOut; iNeuron++)
	{
		if(m_cOutputOut(iNeuron) < dTolerance)
			cOutput(iNeuron) = 0.0;
		else if(m_cOutputOut(iNeuron) > (1.0 - dTolerance))
			cOutput(iNeuron) = 1.0;
		else
			// Not sure :)
			cOutput(iNeuron) = 0.5;
	}
}

/*
void CForwardNetwork::Learn(
	const double dAlpha,
	const double dMomentum,
//	CTrainSet &TrainSet,
	const double dTolerance,
	const double dPrecision,
	const int iMaxStage,
	const bool bProgress
)
{
	// Indexes
	int iPattern;
	int iNeuron;
	int iInput;
	int iStage = 1;
	// Current output, output sigma, hidden sigma 
	CVector Outputs(m_iSizeOut);
	CVector SigmaOut(m_iSizeOut);
	CVector SigmaHid(m_iSizeHid);
	// Cumulative error, ...
	double dError;
	double dDiff; 
	double dNorm;
	// Number of correct classifications
	int iCorrect;
	int iEqual;

	while(iStage <= iMaxStage)
	{
		TrainSet.Scramble();
		
		// Learning stage through all the patterns
		for(iPattern = 0; iPattern < TrainSet.GetSize(); iPattern++)
		{
			Process(TrainSet(iPattern).Inputs, Outputs, dTolerance);

			// Delta for network output
			for(iNeuron = 0; iNeuron < m_iSizeOut; iNeuron++)
				SigmaOut(iNeuron) = DiffF(m_OutputOut(iNeuron)) *
					(TrainSet(iPattern).Desired(iNeuron) - m_OutputOut(iNeuron));

			// Delta for the hidden layer
			for(iNeuron = 0; iNeuron < m_iSizeHid; iNeuron++)
			{
				SigmaHid(iNeuron) = 0.0;
				for(iInput = 0; iInput < m_iSizeOut; iInput++)
					SigmaHid(iNeuron) += 
						SigmaOut(iInput) * m_WeightsOut(iInput, iNeuron);
				SigmaHid(iNeuron) *= DiffF(m_OutputHid(iNeuron));
			}

			// Update output weights
			for(iNeuron = 0; iNeuron < m_iSizeOut; iNeuron++)
				for(iInput = 0; iInput < m_iSizeHid; iInput++)
				{
					m_DeltaOut(iNeuron, iInput) *= rMomentum; 
					m_DeltaOut(iNeuron, iInput) += 
						rAlpha * SigmaOut(iNeuron) * m_OutputHid(iInput);
					m_WeightsOut(iNeuron, iInput) += m_DeltaOut(iNeuron, iInput);
				}
			
			// Update hidden weights
			for(iNeuron = 0; iNeuron < m_iSizeHid; iNeuron++)
				for(iInput = 0; iInput < m_iSizeIn; iInput++)
				{
					m_DeltaHid(iNeuron, iInput) *= rMomentum;
					m_DeltaHid(iNeuron, iInput) +=
						rAlpha * SigmaHid(iNeuron) * TrainSet(iPattern).Inputs(iInput);
					m_cWeightsHid(iNeuron, iInput) += m_DeltaHid(iNeuron, iInput);
				}
		}

		// Count the cumulative error from all the train patterns
		rError = 0.0;
		iCorrect = 0;
		for(iPattern = 0;	iPattern < TrainSet.GetSize(); iPattern++)
		{
			Process(TrainSet(iPattern).Inputs, Outputs, dTolerance);

			// Difference from desired
			rNorm = 0.0;
			iEqual = 1;
			for(iNeuron = 0; iNeuron < m_iSizeOut; iNeuron++)
			{
				// Check whether the fixed output was correct
				if(TrainSet(iPattern).Desired(iNeuron) != Outputs(iNeuron))
					iEqual = 0;

				// Count error from continuos network output
				rDiff = TrainSet(iPattern).Desired(iNeuron) - m_OutputOut(iNeuron);
				rNorm += rDiff * rDiff;
			}
			// Update the error function & number of guest results
			rError += 0.5 * rNorm;
			iCorrect += iEqual;
		}
		
		if(iStage % 10 == 0)
		{
			// Show the progress
			if(bProgress)
				fprintf(
					stderr, 
					"\rLearning stage: %5d Error: %6.3f #: %2d ",
					iStage, rError, iCorrect
				);

			printf(
				"\nLearning stage: %d\n"
				" - Cumulative error: %f\n"
				" - # of correctly clasified patterns: %d\n",
				iStage, rError, iCorrect
			);
		}

		if(rPrecision * TrainSet.GetSize() < iCorrect)
		{
			iStage++;
			// Learning finished earlier
			break;
		}
		
		// Next stage
		iStage ++;
	}

	// Show the progress
	if(bProgress)
		fprintf(
			stderr, 
			"\rLearning stage: %5d Error: %6.3f #: %2d ",
			iStage - 1, rError, iCorrect
		);

	printf("\nLearning finished after %d stages\n", iStage - 1);
	printf(
		"\nLearning stage: %d\n"
		" - Cumulative error: %f\n"
		" - # of correctly clasified patterns: %d\n",
		iStage - 1, rError, iCorrect
	);

}
*/
