//*****************************************************************************
// Layer.cpp: Implementation of the CLayer class
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CLayer::CLayer(
	const int iSizeIn,
	const int iSizeOut,
	CActivate &cActivate
)
:	// Weights matrices
	m_cWeights(iSizeOut, iSizeIn, -1.0, 1.0),
	m_cDiffW(iSizeOut, iSizeIn),
	// Limits used
	m_iSizeIn(iSizeIn),
	m_iSizeOut(iSizeOut),
	// Activation function
	m_cActivate(cActivate)
{
	puts(" - Weights: ");
	m_cWeights.Print(" % 6.4f");
	puts("");
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

void CLayer::OutputDelta(
	CVector &cDesired,
	CVector &cOutput,
	CVector &cDelta
)
{
	// Indexes
	int iNeuron;

	// Delta for network output
	for(iNeuron = 0; iNeuron < m_iSizeOut; iNeuron++)
		cDelta(iNeuron) = (cDesired(iNeuron) - cOutput(iNeuron)) *
			m_cActivate.DiffF(cOutput(iNeuron));
}

void CLayer::HiddenDelta(
	CVector &cDeltaNext,
	CVector &cOutput,
	CVector &cDelta
)
{
	// Indexes
	int iNeuron;
	int iInput;

	// Delta for the hidden layer
	for(iNeuron = 0; iNeuron < m_iSizeIn; iNeuron++)
	{
		cDelta(iNeuron) = 0.0;
		for(iInput = 0; iInput < m_iSizeOut; iInput++)
			cDelta(iNeuron) += cDeltaNext(iInput) * m_cWeights(iInput, iNeuron);
		cDelta(iNeuron) *= m_cActivate.DiffF(cOutput(iNeuron));
	}
}

void CLayer::Adapt(
	const double dAlpha,
	const double dMomentum,
	CVector &cDelta,
	CVector &cInput
)
{
	// Indexes
	int iNeuron;
	int iInput;

	// Update weights
	for(iNeuron = 0; iNeuron < m_iSizeOut; iNeuron++)
	{
		for(iInput = 0; iInput < m_iSizeIn; iInput++)
		{
			m_cDiffW(iNeuron, iInput) *= dMomentum; 
			m_cDiffW(iNeuron, iInput) += 
				dAlpha * cDelta(iNeuron) * cInput(iInput);
			m_cWeights(iNeuron, iInput) += m_cDiffW(iNeuron, iInput);
		}
	}
}

void CLayer::Process(
	CVector &cInput,
	CVector &cOutput
)
{
	// Indexes
	int iNeuron;
	int iInput;
	double dNet;
	
	// Threshold weight
	cInput(m_iSizeIn - 1) = -1.0;
	// Ouptut from hidden layer
	for(iNeuron = 0; iNeuron < m_iSizeOut; iNeuron++)
	{
		dNet = 0.0;
		for(iInput = 0; iInput < m_iSizeIn; iInput++)
			dNet += cInput(iInput) * m_cWeights(iNeuron, iInput);
		cOutput(iNeuron) = m_cActivate.F(dNet);
	}
}
