//*****************************************************************************
// Matrix.cpp: Implementation of the CMatrix class
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CRandom CMatrix::m_cInitVal;

CMatrix::CMatrix(
	const int iHight, 
	const int iWidth,
	const double dValue // = 0.0
)
:	m_iHight(0),
	m_iWidth(0),
	m_pdStorage(NULL)
{
	int iIndex;
	int iLength = iHight * iWidth;

	m_pdStorage = new double[iLength];
	for(iIndex = 0; iIndex < iLength; iIndex++)
		m_pdStorage[iIndex] = dValue;

	m_iWidth = iWidth;
	m_iHight = iHight;
}

CMatrix::CMatrix(CMatrix &Source)
:	m_iHight(0),
	m_iWidth(0),
	m_pdStorage(NULL)
{
	int iLength = Source.m_iHight * Source.m_iWidth;
	m_pdStorage = new double[iLength];
	memcpy(m_pdStorage, Source.m_pdStorage, iLength * sizeof(double));

	m_iHight = Source.m_iHight;
	m_iWidth = Source.m_iWidth;
}

CMatrix::CMatrix(
	const int iHight,
	const int iWidth,
	const double dBottom,
	const double dTop
)
:	m_iHight(0),
	m_iWidth(0),
	m_pdStorage(NULL)
{
	double dScope = dTop - dBottom;
	int iLength = iHight * iWidth;
	int iIndex;

	m_pdStorage = new double[iLength];
	for(iIndex = 0; iIndex < iLength; iIndex ++)
		m_pdStorage[iIndex] = dBottom + dScope * m_cInitVal.Random();

	m_iHight = iHight;
	m_iWidth = iWidth;
}

CMatrix::~CMatrix() 
{
	if(m_pdStorage != NULL)
	{
		delete m_pdStorage;
		m_pdStorage = NULL;
	}
	m_iWidth = m_iHight = 0;
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

void CMatrix::Print(const char *sFormat)
{
	int iLength = m_iWidth * m_iHight;
	int iIndex;

	for(iIndex = 0; iIndex < iLength; iIndex ++)
	{
		printf(sFormat, m_pdStorage[iIndex]);
		if((iIndex + 1) % m_iWidth == 0)
			putchar('\n');
	}
}

double &CMatrix::operator ()(const int iRow, const int iColumn)
{
#ifdef CHECK_INDEX
	if(iRow >= m_iHight || iColumn >= m_iWidth || m_pdStorage == NULL)
		return *NULL;
#endif // CHECK_INDEX

	return m_pdStorage[iRow * m_iWidth + iColumn];
}

CMatrix &CMatrix::operator =(CMatrix &cSource)
{
	int iLength = m_iWidth * m_iHight;

#ifdef CHECK_INDEX
	if(m_iWidth != Source.m_iWidth || m_iHight != Source.m_iHight)
		return *NULL;
#endif // CHECK_INDEX

	memcpy(m_pdStorage, cSource.m_pdStorage, iLength * sizeof(double));
	return(*this);
}
