//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Rule.cpp
 *  \brief Implementacia triedy CRule - pravidlo
 *  \author Rado Kostelnik
 *  \date 1.12.2001 1:20:25
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRule::CRule(CRuleEngine* pParent)
:	m_pParent(pParent)
{
}

CRule::~CRule()
{
	TActionVectorIterator pActionIter = cActionList.begin();
	TActionVectorIterator pActionLast = cActionList.end();

	for(; pActionIter != pActionLast; pActionIter++)
		delete (*pActionIter);

	cActionList.clear();
	cCondList.clear();
}

//------------------------------------------------------
/*! \fn	bool CRule::SetXML(IXMLDOMNodePtr &pNode)
 *  \brief Cita udaje pravidla z XML elementu
 *  \param 
 *  \return
 *  \author Rado Kostelnik
 *  \date 1.12.2001 1:13:05
 */
//------------------------------------------------------
/* Historia zmien:
 */
//------------------------------------------------------
KPDL_RESULT CRule::SetXML(IXMLDOMNodePtr &pNode)
{
	_variant_t vValue;
	IXMLDOMNodeListPtr pChildList;
	IXMLDOMNodePtr pChild;
	CCondition cCond;
	CAction *pAction;

	//
	// read attributes
	//
	pNode->attributes->getNamedItem("name")->get_nodeValue(&vValue);
	sName = _bstr_t(vValue);

	//
	// read conditions
	//
	pChildList = pNode->selectNodes(L"conds/cond");
	pChildList->reset();

	while((pChild = pChildList->nextNode()) != NULL)
	{
		cCond.SetXML(pChild);
		cCondList.push_back(cCond);
	}

	//
	// read actions
	//
	pChildList = pNode->selectNodes(L"actions/*");
	pChildList->reset();

	while((pChild = pChildList->nextNode()) != NULL)
	{
		pAction = new CAction();
		pAction->SetXML(pChild);
		cActionList.push_back(pAction);
	}

	return KPDL_SUCCESS;
}

//------------------------------------------------------
/*! \fn	TActionVector* CRule::Evaluate(CQuery& cQuery, TActionVector &cResult)
 *  \brief Vyhodnotenie pravidla
 *  \param cQuery Objekt, ktory vracia hodnoty premennych
 *  \param cResult Vektor akcii a ich parametrov
 *  \return KPDL_SUCCESS vyhodnotenie pravidiel bolo uspesne
 *          KPDL_UNSUCCESS pravidlo sa v strome nenaslo
 *  \author Rado Kostelnik
 *  \date 1.12.2001 1:13:43
 */
//------------------------------------------------------
/* Historia zmien:
 */
//------------------------------------------------------
KPDL_RESULT CRule::Evaluate(CQuery &cQuery, TActionVector &cResult)
{
	cParserLog.Write(
		true, 
		"pravidlo %s:", 
		sName.c_str()
	);

	KPDL_RESULT kpdlResult;
	TConditionVectorIterator pCondIter = cCondList.begin();
	TConditionVectorIterator pCondLast = cCondList.end();

	// 
	// evaluate conditions
	//
	for(; pCondIter != pCondLast; pCondIter++)
	{
		CCondition &cCond = *pCondIter;
		if(!cCond.Evaluate(cQuery))
			return KPDL_UNSUCCESS;
	}

	TActionVectorIterator pActionIter = cActionList.begin();
	TActionVectorIterator pActionLast = cActionList.end();

	//
	// generate the result action list
	//
	for(; pActionIter != pActionLast; pActionIter++)
	{
		CAction *pAction = (*pActionIter);

		//
		// Evaluate the action
		//
		kpdlResult = m_pParent->Evaluate(
			pAction->sName, 
			cQuery, 
			cResult
		);
		
		if(kpdlResult == KPDL_UNSUCCESS)
		{
			cParserLog.Write(
				true,
				"    -> akcia '%s'",
				pAction->sName.c_str()
			);
			// Found an elementary action
			cResult.push_back(pAction);
		}
	}

	return KPDL_SUCCESS;
}
