//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file		HigherSkills.cpp
 *  \brief		Implementacia HigherSkills - vyssie schopnosti hraca
 *  \author		Rado Kostelnik
 *  \date		1.12.2001 17:28:47
 *  \version	0.1

 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHigherSkills::CHigherSkills(
	CLocalAgent* pAgent, 
	const string &sFileName
)
:	m_pAgent(pAgent), 
	m_cParamQuery(pAgent),
	m_cDispQuery(pAgent) 
{
	m_cRuleEngine.LoadXML(sFileName);
}

CHigherSkills::~CHigherSkills()
{
}

//------------------------------------------------------
/*! \fn			void CHigherSkills::Deliberate()
 *  \brief		Rozhodovanie o nasledujucej cinnosti na vyssej urovni
 *  \param		
 *  \return		nic
 *  \author		Rado Kostelnik
 *  \date		1.12.2001 17:16:09

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
void CHigherSkills::Deliberate()
{
	TActionVector cActionList;
	string sResult;
	char sBuffer[100];
	int iPriority = 0;
	TParameterBSkillsVector cParamList;
	CBasicSkill* pSkill = NULL;
	CBasicSkillList* pSkillList = m_pAgent->listSkl;  

	// Novy rozhodovaci cyklus
	cParserLog.Write(
		true, 
		"------------------------------------------------------------"
	);

	m_cRuleEngine.Evaluate("default", m_cDispQuery, cActionList);

	TActionVectorIterator pActionIter = cActionList.begin();
	TActionVectorIterator pActionEnd = cActionList.end();

	for(; pActionIter != pActionEnd; pActionIter++ )
	{
		CAction *pAction = (*pActionIter);

		//
		// Vyhodnotenie realnych hodnot parametrov akcii
		//
		TRuleParameterVectorIterator pParamIter = pAction->cParamList.begin();
		TRuleParameterVectorIterator pParamEnd = pAction->cParamList.end();

		cParamList.clear();
		for(; pParamIter != pParamEnd; pParamIter++)
		{
			CRuleParameter& cParam = (*pParamIter);
			
			long lPos;

			// location
			if(cParam.sType == "location")
			{
				sResult = m_cParamQuery.Evaluate(cParam.sValue);
				lPos = sResult.find(" ");
				cParamList.push_back(CParameter(VALUE_X, sResult.substr(0, lPos)));
				cParamList.push_back(CParameter(VALUE_Y, sResult.substr(lPos + 1)));
			}
			// playerid
			else if(cParam.sType == "playerid")
			{
				sResult = m_cParamQuery.Evaluate(cParam.sValue);
				cParamList.push_back(CParameter(VALUE_PLAYER, sResult));
			}
			// effort
			else if(cParam.sType == "effort")
			{
				if(cParam.sValue == "Maximum")
					iPriority = 3;
				else if(cParam.sValue == "Medium")
					iPriority = 2;
				else if(cParam.sValue == "Minimum")
					iPriority = 1;
			}
			// angle
			else if(cParam.sType == "angle")
			{
				cParamList.push_back(CParameter(VALUE_ANGLE, cParam.sValue));
			}
			else
				cSystemLog.Write(
					true, 
					"Chyba pri vyhodnocovani parametrov: neznamy typ '%s'" ,
					cParam.sType.c_str()
				);
		}

		//
		// Kick
		//
		if(pAction->sName == "Kick")
		{
			pSkill = new CKick(cParamList, m_pAgent, iPriority);
		}
		else
		//
		// GoToPosition
		//
		if(pAction->sName == "GoToPosition")
		{
			itoa(BSKILL_MOVE_GOTO, sBuffer, 10);
			cParamList.push_back(CParameter(VALUE_TYPE, sBuffer));
			pSkill = new CMove(cParamList, m_pAgent, iPriority);
		}
		else 
		//
		// DribbleTo
		//
		if(pAction->sName == "DribbleTo")
		{
			pSkill = new CDribble(cParamList, m_pAgent, iPriority);
		}
		else 
		//
		// TurnTo
		//
		if(pAction->sName == "Turn")
		{
			pSkill = new CTurn(cParamList, m_pAgent, iPriority);
		}
		else
		//
		// PassTo
		//
		if(pAction->sName == "PassTo")
		{
			pSkill = new CPass(cParamList, m_pAgent, iPriority);
		}
		else 
		//
		// Catch
		//
		if(pAction->sName == "Catch")
		{
			pSkill = new CCatch(cParamList, m_pAgent, iPriority);
		}
		else 
		//rado 27. 12. 2001
		//
		// GoToBallPosition
		//
		if(pAction->sName == "GoToBallPosition") 
		{
			itoa(BSKILL_MOVE_GOTO_BALL, sBuffer, 10);
			cParamList.push_back(CParameter(VALUE_TYPE, sBuffer));
			pSkill = new CMove(cParamList, m_pAgent, iPriority);
		}

		if(pSkill != NULL)
		{
			pSkillList->AddSkill(pSkill);
			cSystemLog.Write(
				true,
				"pridana akcia: '%s'", 
				pAction->sName.c_str()
			);
		}
		pSkill = NULL;

	}
}