//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	wodispatch.cpp
 *  \brief Implementacia Dispatch interface objektov World Modelu
 *  \author Rado Kostelnik
 *  \date 27.2.2002 19:58:38
 *  \version 

 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdAfx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX_BALL_CONTROL_DIST  5      // krokov
#define MAX_BALL_DIST          50.0   // metrov
#define MAX_OPPONENT_DIST      10.0   // metrov
#define MAX_LAST_SEEN          4      // cykly
#define KPDL_TRUE              "1.0"
#define KPDL_FALSE             "0.0"

//======================================================
// CBall : CDispatch implementacia
//======================================================
CDispatch* CBall::getDispatch( string sName )
{
	return NULL;
}

string CBall::getAttribute( string sName )
{
	string sRetValue;
	CMovingObject* me = (CMovingObject*) this;
	CDiffuse2DVector v;
	char buf[100];
	double dValue = 0.0;

	sRetValue = "";

	if ( sName == "speed" )
	{
		v = me->GetSpeed();
		sprintf( buf, "%lf %lf", v.GetX(), v.GetY() );

		sRetValue = buf;
	}
	else if ( sName == "position" )
	{
		v = me->GetPosition();
		sprintf( buf, "%lf %lf", v.GetX(), v.GetY() );

		sRetValue = buf;
	}
	else if ( sName == "direction" )
	{
		sprintf( buf, "%lf", GetDirection() );

		sRetValue = buf;
	}

	return sRetValue;
}


//======================================================
// CLocalAgent : CDispatch implementacia
//======================================================
CDispatch* CLocalAgent::getDispatch( string sName )
{
	string sRetValue;
	CDispatch* d = NULL;

	//-------------------------------------------------------
	// me (reflexia)
	//-------------------------------------------------------
	if ( sName == "me" )
	{
		d = (CDispatch*) this;
	}
	else if ( sName == "strategy" )
	{
		d = (CDispatch*) strategicSkills;
	}
	
	return d;
}

string CLocalAgent::getAttribute( string sName )
{
	string sResult;
	char buf[100];
	CMovingObject* me = (CMovingObject*) this;
	CDiffuse2DVector v;

	sResult = "";

	//-------------------------------------------------------
	// haveBall
	//-------------------------------------------------------
	if ( sName == "haveBall" )
	{
		int iPlayerCount = GetWorldModel()->GetIdentifiedAgentsCount();
		CAgent *pOpponent = NULL;
    
		double dBallDist = bskl->RelVectorTo(
			this, 
			GetWorldModel()->GetBall()->GetPosition()
		).GetDistance();

		double dOpponentBallDist;
		double dBestOpponentBallDist = MAX_BALL_DIST;
		double dStepDist;
		double dOpponentStepDist;

		if(dBallDist < cfg->kickable_area)
			return KPDL_TRUE;

		// Vzdialenost lopty od superov
		for(int i=0; i<iPlayerCount; i++)
		{
			pOpponent = GetWorldModel()->GetIdentifiedAgent(i);
			//bskl->BallPosition(m_pAgent);
			//bskl->GetBallInterceptPosition()
			if(pOpponent->GetPlayerSide() == S_OPPONENT)
			{
				dOpponentBallDist = bskl->RelVectorTo(
					pOpponent, 
					GetWorldModel()->GetBall()->GetPosition()
				).GetDistance();

				if(dBestOpponentBallDist < dOpponentBallDist)
					dBestOpponentBallDist = dOpponentBallDist;
			}
		}

		// Dalej predpokladajme, ze hrac sa bude k lopte priblizovat priamo,
		// maximalnou rychlostou
		dStepDist = dBallDist / cfg->player_speed_max;
		dOpponentStepDist = dBestOpponentBallDist / cfg->player_speed_max;

		// Ak je blizsie super
		if(dOpponentStepDist < dStepDist && 
			dOpponentStepDist < MAX_BALL_CONTROL_DIST)
			return KPDL_FALSE;

		// Ak je blizsie hrac
		if(dOpponentStepDist > dStepDist && 
			dStepDist < MAX_BALL_CONTROL_DIST)
			return KPDL_TRUE;

		// nejaka funkcia vhodne ohodnocujuca zvysne pripady
		sprintf( buf, "%lf", (dOpponentStepDist / (dOpponentStepDist + dStepDist)) );

		sResult = buf;
		
		// ! mozno vhodne pridat este clen - cim dalej je lopta od hraca,
		// tym mensia kontrola
	}
	//-------------------------------------------------------
	// nearGoal
	//-------------------------------------------------------
	else if ( sName == "nearGoal" )
	{
		double res = 
			((GetPosition().GetX() - 
			  field->GetGoal(GT_HOME).GetX()) / 
			 (field->GetFieldLength()));
		sprintf(buf,"%lf",res);
		sResult = buf;
	}
	//-------------------------------------------------------
	// freePathToGoal
	//-------------------------------------------------------
	else if ( sName == "freePathToGoal" )
	{
		/*
		C2DVector cUpperGoalFlag = m_pAgent->field->GetGoal(GT_HOME);
		C2DVector cLowerGoalFlag = m_pAgent->field->GetGoal(GT_HOME);
		CAgent *pOpponent = NULL;

		cUpperGoalFlag.SetY(- m_pAgent->field->GetGoalSize() / 2.0);
		cLowerGoalFlag.SetY(+ m_pAgent->field->GetGoalSize() / 2.0);
		*/
		sResult = "0.9";
	}
	//-------------------------------------------------------
	// seeBall
	//-------------------------------------------------------
	else if ( sName == "seeBall" )
	{
		double dPI2 = PI / 2;
		double dBallAngle = GetWorldModel()->GetBall()->GetSpeed().GetAngle();
		
		// ! Dufam, ze last seen je pocet cyklov, kedy bola lopta naposledy videna, nie
		// je to uplne korektny test ...
		// Rado Otipka 16. 12. 2001 15:27:02
		//if(dBallAngle > -dPI2 && dBallAngle < dPI2 && 
		//	m_pAgent->GetWorldModel()->GetBall()->GetLastSeen() < MAX_LAST_SEEN)
		//	return KPDL_TRUE;
		if( GetTime().m_nPlayTime 
			- GetWorldModel()->GetBall()->m_cLastVisible.m_nPlayTime  > MAX_LAST_SEEN ) 
			sprintf ( buf, "%ld", 0 );
		else
			if(dBallAngle > -dPI2 && dBallAngle < dPI2 )
				sprintf ( buf, "%ld", 1 );
		
			// Rado Otipka 16. 12. 2001 15:27:06
			else
				sprintf ( buf, "%ld", 0 );
		sResult = buf;
	}
	//-------------------------------------------------------
	// ballKickable
	//-------------------------------------------------------
	else if ( sName == "ballKickable" )
	{
		// Rado Otipka 16. 12. 2001 12:52:27 begin
		if( GetTime().m_nPlayTime - GetWorldModel()->GetBall()->m_cLastVisible.m_nPlayTime > 2) 
		{
			return string("0.0");
		}

		// Rado Otipka 16. 12. 2001 12:52:27 end
		double dBallDist = bskl->RelVectorTo(
			this, 
			GetWorldModel()->GetBall()->GetPosition()
		).GetDistance();

		if (dBallDist < cfg->kickable_area)
			return string("1.0");

		return string("0.0");
	}
	//-------------------------------------------------------
	// ballNear
	//-------------------------------------------------------
	else if ( sName == "ballNear" )
	{
		double dBallDist = bskl->RelVectorTo(
			this, 
			GetWorldModel()->GetBall()->GetPosition()
		).GetDistance();

		double dNormBallDist = dBallDist / MAX_BALL_DIST;
		
		if(dNormBallDist < 1.0)
		{
			sprintf( buf, "%lf", (1.0 - dNormBallDist) );
			sResult = buf;
			return sResult;
		}
		
		// Ball is too far
		return string("0.0");
	}
	//-------------------------------------------------------
	// amIFree
	//-------------------------------------------------------
	else if ( sName == "amIFree" )
	{
		double dOpponentDist;
		double dNormOpponentDist;
		double dBestNormOpponentDist = 1.0;
		int iPlayerCount = GetWorldModel()->GetIdentifiedAgentsCount();
		CAgent *pOpponent = NULL;

		for(int i=0; i<iPlayerCount; i++)
		{
			pOpponent = GetWorldModel()->GetIdentifiedAgent(i);

			if(pOpponent->GetPlayerSide() == S_OPPONENT)
			{
				dOpponentDist = bskl->RelVectorTo(
					this, 
					pOpponent->GetPosition()
				).GetDistance();

				dNormOpponentDist = dOpponentDist / MAX_OPPONENT_DIST;

				if(dBestNormOpponentDist < dNormOpponentDist)
					dBestNormOpponentDist = dNormOpponentDist;
			}
		}

		sprintf ( buf, "%lf", (1.0 - dBestNormOpponentDist) );
		sResult = buf;
	}
	//-------------------------------------------------------
	// teamMateHasBall
	//-------------------------------------------------------
	else if ( sName == "teamMateHasBall" )
	{
		int iPlayerCount = GetWorldModel()->GetIdentifiedAgentsCount();
		CAgent *pTeamMate = NULL;
		CAgent *pOpponent = NULL;

		double dBallDist;
		double dOpponentBallDist;
		double dBestOpponentBallDist = MAX_BALL_DIST;
		double dStepDist;
		double dOpponentStepDist;
		double dResult;
		double dBestResult = 0.0; // pesimistic view

		for(int i=0; i<iPlayerCount; i++)
		{
			pTeamMate = GetWorldModel()->GetIdentifiedAgent(i);

			dBallDist = bskl->RelVectorTo(
				this, 
				GetWorldModel()->GetBall()->GetPosition()
			).GetDistance();
			
			if(dBallDist < cfg->kickable_area)
				return string("1.0");

			// Dalej predpokladajme, ze hrac sa bude k lopte priblizovat priamo,
			// maximalnou rychlostou
			dStepDist = dBallDist / cfg->player_speed_max;

			if(pTeamMate->GetPlayerSide() == S_HOME &&
				pTeamMate->GetLocalNumber() != GetLocalNumber())
			{
				// Vzdialenost lopty od superov
				for(int i=0; i<iPlayerCount; i++)
				{
					pOpponent = GetWorldModel()->GetIdentifiedAgent(i);

					if(pOpponent->GetPlayerSide() == S_OPPONENT)
					{
						dOpponentBallDist = bskl->RelVectorTo(
							pOpponent, 
							GetWorldModel()->GetBall()->GetPosition()
						).GetDistance();

						if(dBestOpponentBallDist < dOpponentBallDist)
							dBestOpponentBallDist = dOpponentBallDist;
					}
				}

				dOpponentStepDist = dBestOpponentBallDist / cfg->player_speed_max;

				// Ak je blizsie super
				if(dOpponentStepDist < dStepDist && 
					dOpponentStepDist < MAX_BALL_CONTROL_DIST)
				{
					dResult = 0.0;
					break;
				}

				// Ak je blizsie spoluhrac
				if(dOpponentStepDist > dStepDist && 
					dStepDist < MAX_BALL_CONTROL_DIST)
				{
					dResult = 1.0;
					break;
				}

				// nejaka funkcia vhodne ohodnocujuca zvysne pripady
				dResult = dOpponentStepDist / (dOpponentStepDist + dStepDist);
				// ! mozno vhodne pridat este clen - cim dalej je lopta od hraca,
				// tym mensia kontrola
			}

			// vezmeme najlepsi vysledok
			if(dBestResult < dResult)
				dBestResult = dResult;
		}

		sprintf( buf, "%lf", dBestResult );
		sResult = buf;
		return sResult;
	}	
	//-------------------------------------------------------
	// teamMateHasBall
	//-------------------------------------------------------
	else if ( sName == "teamMateNearBall" )
	{
		double dBallDist;
		double dNormBallDist;
		double dBestNormBallDist = 1.0;
		int iPlayerCount = GetWorldModel()->GetIdentifiedAgentsCount();
		CAgent *pTeamMate = NULL;

		for(int i=0; i<iPlayerCount; i++)
		{
			pTeamMate = GetWorldModel()->GetIdentifiedAgent(i);

			if(pTeamMate->GetPlayerSide() == S_HOME &&
				pTeamMate->GetLocalNumber() != GetLocalNumber())
			{
				dBallDist = bskl->RelVectorTo(
					pTeamMate, 
					GetWorldModel()->GetBall()->GetPosition()
				).GetDistance();

				dNormBallDist = dBallDist / MAX_BALL_DIST;

				if(dBestNormBallDist > dNormBallDist)
					dBestNormBallDist = dNormBallDist;
			}
		}

		sprintf( buf, "%lf", (1.0 - dBestNormBallDist) );
		sResult = buf;
	}	
	//-------------------------------------------------------
	// canIGetPass
	//-------------------------------------------------------
	else if ( sName == "canIGetPass" )
	{
		double dOpponentDist;
		double dNormOpponentDist;
		double dBestNormOpponentDist = 1.0;
		int iPlayerCount = GetWorldModel()->GetIdentifiedAgentsCount();
		CAgent *pOpponent = NULL;

		for(int i=0; i<iPlayerCount; i++)
		{
			pOpponent = GetWorldModel()->GetIdentifiedAgent(i);

			if(pOpponent->GetPlayerSide() == S_OPPONENT)
			{
				dOpponentDist = bskl->RelVectorTo(
					this, 
					pOpponent->GetPosition()
				).GetDistance();

				dNormOpponentDist = dOpponentDist / MAX_OPPONENT_DIST;

				if(dBestNormOpponentDist < dNormOpponentDist)
					dBestNormOpponentDist = dNormOpponentDist;
			}
		}

		// ! ak som volny, treba este skontrolovat, ci medzi mnou a spoluhracom s loptou
		// je volna cesta
		sprintf( buf, "%lf",(1.0 - dBestNormOpponentDist));
		sResult = buf;
	}
	//-------------------------------------------------------
	// someoneHasBall
	//-------------------------------------------------------
	else if ( sName == "someOneHasBall" )
	{
		int i;
		int iPlayerCount = GetWorldModel()->GetIdentifiedAgentsCount();
		CAgent *pPlayer = NULL;

		double dBallSpeed = GetWorldModel()->GetBall()->GetSpeed().GetDistance();
		double dBallDist;
		double dStepDist;
		double dBestStepDist = MAX_BALL_DIST; 
		
		for(i=0; i<iPlayerCount; i++)
		{
			pPlayer = GetWorldModel()->GetIdentifiedAgent(i);

			dBallDist = bskl->RelVectorTo(
				pPlayer, 
				GetWorldModel()->GetBall()->GetPosition()
			).GetDistance();

			// ! Zakomponovat do tohto vypoctu aj rychlost lopty, resp
			// vypocet timu cislo 4, o aky cas dobehne hrac loptu ...
			dStepDist = dBallDist / cfg->player_speed_max;

			if(dBallDist < cfg->kickable_area ||
				dStepDist < MAX_BALL_CONTROL_DIST)
				return string(KPDL_TRUE);

			// vezmeme najlepsi vysledok
			if(dBestStepDist < dBallDist)
				dBestStepDist = dBallDist;
		}

		iPlayerCount = GetWorldModel()->GetUnIdentifiedAgentsCount();
		for(i=0; i<iPlayerCount; i++)
		{
			pPlayer = GetWorldModel()->GetUnIdentifiedAgent(i);

			dBallDist = bskl->RelVectorTo(
				pPlayer, 
				GetWorldModel()->GetBall()->GetPosition()
			).GetDistance();

			// ! Zakomponovat do tohto vypoctu aj rychlost lopty, resp
			// vypocet timu cislo 4, o aky cas dobehne hrac loptu ...
			dStepDist = dBallDist / cfg->player_speed_max;

			if(dBallDist < cfg->kickable_area ||
				dStepDist < MAX_BALL_CONTROL_DIST)
				return string(KPDL_TRUE);

			// vezmeme najlepsi vysledok
			if(dBestStepDist < dBallDist)
				dBestStepDist = dBallDist;
		}

		sprintf( buf, "%lf", (1 - dBestStepDist / MAX_BALL_DIST) );
		sResult = buf;
	}
	//-------------------------------------------------------
	// haveFreeTeamMate
	//-------------------------------------------------------
	else if ( sName == "haveFreeTeamMate" )
	{
		double dOpponentDist;
		double dNormOpponentDist;
		double dWorstNormOpponentDist = 0.0;
		double dBestNormOpponentDist = 1.0;
		int iPlayerCount = GetWorldModel()->GetIdentifiedAgentsCount();
		CAgent *pOpponent = NULL;
		CAgent *pTeamMate = NULL;

		for(int i=0; i<iPlayerCount; i++)
		{
			pTeamMate = GetWorldModel()->GetIdentifiedAgent(i);

			if(pTeamMate->GetPlayerSide() == S_HOME &&
				pTeamMate->GetLocalNumber() != GetLocalNumber())
			{
				dBestNormOpponentDist = 1.0;
				for(int j=0; j<iPlayerCount; j++)
				{
					pOpponent = GetWorldModel()->GetIdentifiedAgent(j);

					if(pOpponent->GetPlayerSide() == S_OPPONENT)
					{
						dOpponentDist = bskl->RelVectorTo(
							pTeamMate, 
							pOpponent->GetPosition()
						).GetDistance();

						dNormOpponentDist = dOpponentDist / MAX_OPPONENT_DIST;

						if(dBestNormOpponentDist < dNormOpponentDist)
							dBestNormOpponentDist = dNormOpponentDist;
					}
				}

				if(dWorstNormOpponentDist < dBestNormOpponentDist)
					dWorstNormOpponentDist = dBestNormOpponentDist;
			}
		}

		sprintf( buf, "%lf", 1.0 - dWorstNormOpponentDist );
		sResult = buf;
	}
	//-------------------------------------------------------
	// haveBetterLocatedTeamMate
	//-------------------------------------------------------
	else if ( sName == "haveBetterLocatedTeamMate" )
	{
		return string("0.0");
	}
	//-------------------------------------------------------
	// canIPass
	//-------------------------------------------------------
	else if ( sName == "canIPass" )
	{
		double dOpponentDist;
		double dNormOpponentDist;
		double dWorstNormOpponentDist = 0.0;
		double dBestNormOpponentDist = 1.0;
		int iPlayerCount = GetWorldModel()->GetIdentifiedAgentsCount();
		CAgent *pOpponent = NULL;
		CAgent *pTeamMate = NULL;

		for(int i=0; i<iPlayerCount; i++)
		{
			pTeamMate = GetWorldModel()->GetIdentifiedAgent(i);

			if(pTeamMate->GetPlayerSide() == S_HOME &&
				pTeamMate->GetLocalNumber() != GetLocalNumber())
			{
				dBestNormOpponentDist = 1.0;
				for(int j=0; j<iPlayerCount; j++)
				{
					pOpponent = GetWorldModel()->GetIdentifiedAgent(j);

					if(pOpponent->GetPlayerSide() == S_OPPONENT)
					{
						dOpponentDist = bskl->RelVectorTo(
							pTeamMate, 
							pOpponent->GetPosition()
						).GetDistance();

						dNormOpponentDist = dOpponentDist / MAX_OPPONENT_DIST;

						if(dBestNormOpponentDist < dNormOpponentDist)
							dBestNormOpponentDist = dNormOpponentDist;
					}
				}

				if(dWorstNormOpponentDist < dBestNormOpponentDist)
				{
					// ! Tu treba este skontrolovat, nakolko je volna draha lopty
					// k najdenemu spoluhracovi
					dWorstNormOpponentDist = dBestNormOpponentDist;
				}
			}
		}

		sprintf( buf, "%lf", 1.0 - dWorstNormOpponentDist );
		sResult = buf;
	}
	//-------------------------------------------------------
	// stamina
	//-------------------------------------------------------
	else if ( sName == "stamina" )
	{
		sprintf( buf, "%lf", GetStamina() );

		sResult = buf;
	}
	//-------------------------------------------------------
	// viewAngle
	//-------------------------------------------------------
	else if ( sName == "viewAngle" )
	{
		sprintf( buf, "%lf", GetViewAngle() );

		sResult = buf;
	}
	return sResult;
}

