//*****************************************************************************
// Main.cpp : Defines the entry point for the application
//*****************************************************************************

#include "stdafx.h"

int main(int argc, char* argv[])
{
	try
	{
		// One hot transformation of input and output symbols
		COneHot OneHotIn;
		COneHot OneHotOut;
		// Configuration stream
		CStream CfgStream;
		// Input, output and desired sequence of symbols
		CList Input;
		CList Output;
		CHAR sInput[MAX_BUFFER];
		CHAR sOutput[MAX_BUFFER];
		CHAR sDesired[MAX_BUFFER];
		// Number of neurons in input, recurent and output layer
		INT iSizeRec = (INT) 0;
		// Learning rate, momentum
		REAL rAlpha = (REAL) 0.0;
		REAL rMomentum = (REAL) 0.0;
		// Output tolerance, learning precision;
		REAL rTolerance = (REAL) 0.0;
		REAL rErrorLimit = (REAL) 0.0;
		// Number of train patterns; test patterns
		INT iTrainCount = (INT) 0;
		INT iTestCount = (INT) 0;
		// Temporals
		INT iSizeIn = (INT) 0;
		INT iSizeOut = (INT) 0;
		INT iPattern;
		CHAR cCorrect;

		CfgStream.Attach(stdin);

		puts("Loading configuration ...");

		CfgStream >> iSizeRec
				  >> rAlpha
				  >> rMomentum
				  >> rTolerance
				  >> rErrorLimit
				  >> iTrainCount
				  >> iTestCount;
		
		puts(" - One hot input encoding");
		OneHotIn.Load(CfgStream);
		iSizeIn = OneHotIn.GetVectorSize();
		puts(" - One hot output encoding");
		OneHotOut.Load(CfgStream);
		iSizeOut = OneHotOut.GetVectorSize();

		printf(
			" - Input size = %d\n"
			" - Recurent size = %d\n"
			" - Output size = %d\n"
			" - Learning rate = %4.2f\n"
			" - Momentum = %4.2f\n"
			" - Output tolerance = %4.2f\n"
			" - Total error limit = %4.2f\n"
			" - Train pattern count = %d\n"
			" - Test pattern count = %d\n",
			iSizeIn,
			iSizeRec,
			iSizeOut,
			rAlpha,
			rMomentum,
			rTolerance,
			rErrorLimit,
			iTrainCount,
			iTestCount
		);

		CTrainSet TrainSet(iTrainCount,	OneHotIn, OneHotOut);

		puts("\nLoading training set ...");
		TrainSet.Load(CfgStream);
		TrainSet.Print();

		puts("\nCreating neural network ...");
		CContinuousUnipol ContUni;
		CRecurentNetwork Network(
			iSizeIn,
			iSizeRec,
			iSizeOut,
			ContUni,
			rMomentum,
			rAlpha
		);

		puts("\nLearning neural network ...");
		Network.Learn(
			TrainSet,
			rErrorLimit,
			rTolerance
		);

		puts("\nTesting neural network ...");
		for(iPattern = (INT) 0; iPattern < iTestCount; iPattern++)
		{
			CfgStream >> sInput;
			OneHotIn.Encode(sInput, Input);
			CfgStream >> sDesired;

			Network.Process(Input, Output);

			OneHotOut.Decode(
				rTolerance,
				Output,
				sOutput,
				MAX_BUFFER
			);
			cCorrect = (!strcmp(sDesired, sOutput))? ' ' : '*';

			printf(
				" - Pattern: %s  Desired: %s  Output: %s%c\n",
				sInput,
				sDesired,
				sOutput,
				cCorrect
			);
		}
	}
	catch(CException &E)
	{
		putc('\n', stderr);
		fputs(E.GetMessage(), stderr);
	}
	
	// Finaly :) exit program
	return 0;
}
