//*****************************************************************************
// OneHot.cpp: Implementation of the COneHot class
//*****************************************************************************

#include "stdafx.h"

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

VOID COneHot::SetAlphabet(
	const STR sAlphabet
)
{
	INT iSize = (INT) 0;
	
	if(sAlphabet != NULL)
		iSize = strlen(sAlphabet);

	if(m_sAlphabet != NULL)
	{
		delete m_sAlphabet;
		m_sAlphabet = NULL;
	}
	if((m_iSize = iSize) == (INT) 0)
		return;

	m_sAlphabet = new CHAR[iSize + 1];
	strcpy(m_sAlphabet, sAlphabet);
}

VOID COneHot::Encode(
	const CHAR cInput,
	CVector *&pOutput
)
{
	CHAR *pcChar = NULL;
	INT iHot;

	if(m_iSize == (INT) 0)
		throw CException(NO_TRANSTABLE);

	if((pcChar = strchr(m_sAlphabet, cInput)) == NULL)
		throw CException(INVALID_SYMBOL);

	iHot = (pcChar - m_sAlphabet);
	pOutput = new CVector(m_iSize);
	(*pOutput)(iHot) = (REAL) 1.0;
}

VOID COneHot::Encode(
	const STR sInput,
	CList &Output
)
{
	CVector *pVector = NULL;
	INT iLength = (INT) 0;
	INT iIndex;

	Output.Clear();
	
	if(sInput != NULL)
		iLength = strlen(sInput);
	if(iLength == (INT) 0)
		return;

	for(iIndex = (INT) 0; iIndex < iLength; iIndex++)
	{
		Encode(sInput[iIndex], pVector);
		Output.Append(pVector);
	}
}

CHAR COneHot::Decode(
	const REAL rTolerance,
	CVector &Input
)
{
	INT iHotCount = (INT) 0;
	INT iHotIndex;
	INT iIndex;

	for(iIndex = (INT) 0; iIndex < Input.GetSize(); iIndex++)
	{
		if(Input(iIndex) >= (1.0 - rTolerance))
		{
			// Hot found
			if(iHotCount++)
				return UNIDENT_SYMBOL;
			iHotIndex = iIndex;
		}
		else if(Input(iIndex) > rTolerance)
		{
			// Unidentificable input pattern
			return UNIDENT_SYMBOL;
		}
	}

	if(iHotCount == (INT) 0)
		return UNIDENT_SYMBOL;

#ifdef CHECK_INDEX
	if(iHotIndex >= m_iSize)
		throw CException(INVALID_INDEX);
#endif // CHECK_INDEX
	
	return m_sAlphabet[iHotIndex];
}

INT COneHot::Decode(
	const REAL rTolerance,
	CList &Input,
	STR sOutput,
	INT iLength
)
{
	CVector *pVector = NULL;
	INT iIndex = (INT) 0;
	BOOL bWork;
	
	iLength--;
	bWork = Input.MoveFirst();
	while(bWork && iIndex < iLength)
	{
		pVector = (CVector *) Input.Item();
		sOutput[iIndex] = Decode(rTolerance, *pVector);

		bWork = Input.MoveNext();
		iIndex++;
	}

	sOutput[iIndex] = '\0';
	return iIndex;
}
