//*****************************************************************************
// Stream.cpp: Implementation of the CStream class
//*****************************************************************************

#include "stdafx.h"

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CStream::CStream()
:	m_pFile(NULL),
	m_bClose(FALSE)
{
}

CStream::~CStream()
{
	Close();
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

VOID CStream::Attach(FILE *pFile)
{
	// Close previous file if any
	Close();

	if(pFile == NULL)
		throw CException(INVALID_FILEHND);

	m_pFile = pFile;
	m_bClose = FALSE;
}

VOID CStream::Open(const STR strName)
{
	// Close previous file if any
	Close();

	if(strName == NULL)
		throw CException(NO_FILENAME);

	m_pFile = fopen(strName, "tr");
	if(m_pFile == NULL)
		throw CException(ERR_OPEN_FILE);

	m_bClose = TRUE;
}

VOID CStream::Close()
{
	// Reset buffers
	memset(m_strBuffer, '\0', MAX_BUFFER + 1);

	// Close the file if possible
	if(m_bClose && m_pFile != NULL)
		fclose(m_pFile);

	m_pFile = NULL;
}

CStream &CStream::operator >>(CHAR &cInput)
{
	if(!ReadElement(1))
		throw CException(END_OF_FILE);

	cInput = *m_strBuffer;
	return *this;
}

CStream &CStream::operator >>(STR strInput)
{
	if(!ReadElement(MAX_BUFFER))
		throw CException(END_OF_FILE);

	strcpy(strInput, m_strBuffer);
	return *this;
}

CStream &CStream::operator >>(INT &iInput)
{
	if(!ReadElement(MAX_BUFFER))
		throw CException(END_OF_FILE);

	iInput = atoi(m_strBuffer);
	return *this;
}

CStream &CStream::operator >>(REAL &rInput)
{
	if(!ReadElement(MAX_BUFFER))
		throw CException(END_OF_FILE);

	rInput = (REAL) atof(m_strBuffer);
	return *this;
}

//*****************************************************************************
// Helper functions
//*****************************************************************************

BOOL CStream::ReadElement(
	const INT iMaxSize
)
{
	INT iNext = (INT) 0;
	int cChar;

	if(m_pFile == NULL)
		throw CException(FILE_CLOSED);

	// Reset buffer
	strset(m_strBuffer, '\0');

	while(iNext < iMaxSize)
	{
		cChar = fgetc(m_pFile);

		switch(cChar)
		{
		case EOF:
			return (iNext > (INT) 0);

		case ';':
			// Discard comment
			while((cChar = fgetc(m_pFile)) != '\n')
				if(cChar == EOF)
					return (iNext > (INT) 0);

			if(iNext > (INT) 0)
				return TRUE;

			break;

		case ' ':
		case '\t':
		case '\n':
		case '(':
		case ')':
		case '=':
		case '-':
		case '>':
		case '|':
		case ':':
			// Standard delimiter reached
			if(iNext > (INT) 0)
				return TRUE;

			// Delimiters are skipped
			break;
	
		default:
			// Normal character
			m_strBuffer[iNext++] = cChar;
			break;
		}
	}

	return TRUE;
}
