//*****************************************************************************
// TrainSet.cpp: Implementation of the CTrainSet class
//*****************************************************************************

#include "stdafx.h"

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CTrainSet::CTrainSet(
	const INT iSize,
	COneHot &OneHotIn,
	COneHot &OneHotOut
)
:	// Default initialization
	m_iSize((INT) 0),
	m_iNext((INT) 0),
	m_ppStorage(NULL),
	m_OneHotIn(OneHotIn),
	m_OneHotOut(OneHotOut),
	m_Scrambler()
{
	if((m_ppStorage = new CPattern *[iSize]) == NULL)
		throw CException(NO_MEMORY);

	// Remember the size
	m_iSize = iSize;
	memset(m_ppStorage, NULL, iSize * sizeof(CPattern *));
}

CTrainSet::~CTrainSet()
{
	// Destroy the item storage
	if(m_ppStorage != NULL)
	{
		delete [] m_ppStorage;
		m_ppStorage = NULL;
	}
	// Reset the maximal size and the last inserted item
	m_iSize = m_iNext = (INT) 0;
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

VOID CTrainSet::Insert(CPattern *pPattern)
{
	// Check argument
	if(pPattern == NULL)
		throw CException(INVALID_ARG);

	// Check the space
	if(m_iNext >= m_iSize || m_ppStorage == NULL)
		throw CException(NO_MEMORY);

	m_ppStorage[m_iNext++] = pPattern;
}

VOID CTrainSet::Scramble()
{
	CPattern *pTemp;
	INT iFirst;
	INT iSecond;
	INT iScope = m_iNext - 1;

	// Scramble the set of samples
	if(m_ppStorage == NULL)
		throw CException(NO_TRAINSET);

	for(INT iIndex = (INT) 0;
		iIndex < m_iNext;
		iIndex ++
	)
	{
		iFirst = (INT)(iScope * m_Scrambler.Random());
		iSecond = (INT)(iScope * m_Scrambler.Random());

		// Randomly exchange set members
		pTemp = m_ppStorage[iFirst];
		m_ppStorage[iFirst] = m_ppStorage[iSecond];
		m_ppStorage[iSecond] = pTemp;
	}
}

VOID CTrainSet::Print()
{
	CHAR sBuffer[MAX_BUFFER];
	CPattern *pPattern = NULL;
	INT iIndex;

	printf(" - Number of patterns: %d\n", m_iNext);

	for(iIndex = (INT) 0; iIndex < m_iNext; iIndex ++)
	{
		pPattern = m_ppStorage[iIndex];
		// Input sequence
		m_OneHotIn.Decode(
			(REAL) 0.0,
			pPattern->Input,
			sBuffer,
			MAX_BUFFER
		);
		printf(" - Pattern: %s", sBuffer);
		// Desired sequence
		m_OneHotOut.Decode(
			(REAL) 0.0,
			pPattern->Desired,
			sBuffer,
			MAX_BUFFER
		);
		printf(" Desired: %s\n", sBuffer);
		pPattern->Desired.MoveLast();
	}
}

VOID CTrainSet::Load(
	CStream &Stream
)
{
	CHAR sBuffer[MAX_BUFFER];
	CPattern *pPattern = NULL;
	INT iIndex;

	for(iIndex = (INT) 0; iIndex < m_iSize; iIndex++)
	{
		pPattern = new CPattern;
		// Input sequence
		Stream >> sBuffer;
		m_OneHotIn.Encode(sBuffer, pPattern->Input);
		// Desired sequence
		Stream >> sBuffer;
		m_OneHotOut.Encode(sBuffer, pPattern->Desired);
		pPattern->Desired.MoveLast();

		Insert(pPattern);
	}
}
