// WorldObjects.h: interface for the world objects classes.
//
//////////////////////////////////////////////////////////////////////

#ifndef _WORLD_OBJECTS_
#define _WORLD_OBJECTS_

typedef vector<double> doublevector;

typedef vector<CDiffuse2DVector> DiffusePointVector;
typedef vector<C2DVector       > PointVector;
typedef vector<CAngular2DVector> AngularVector;
typedef vector<bool            > boolVector;


class CMovingObject
{
protected:
	double				m_direction;
	CDiffuse2DVector	m_position, m_speed, m_acceleration;
	CDiffuse2DVector	m_prev_position, m_prev_speed, m_prev_acceleration;
	double				m_probability;
	CAngular2DVector	m_see_position, m_see_speed;

	int					m_last_seen;

	DWORD				m_see_status;

    bool				i_am_local;

public:    
    CGameTime			m_cLastVisible;               // FIXME

public:

						CMovingObject()
                        {
                            m_probability=0;
                            m_see_status=0;
                            i_am_local=false;
                        }
    bool				IsValid(CGameTime cNow)
                        {
                            int nDltVisible = cNow.m_nAllCycles-m_cLastVisible.m_nAllCycles;
                            bool bValid = (nDltVisible<=20);
                                
							return bValid;
                        }
    bool				IsVisible(CGameTime cNow)
						{
                            //return (m_see_status&STATE_IS_VISIBLE)!=0;

                            // bude je viditelny, alebo bol naposledy viditelny najviac v minulom kroku
                            int nDltVisible = cNow.m_nAllCycles-m_cLastVisible.m_nAllCycles;
                            //! t0n0
							//fprintf(m_ff, "see_status: %d, dltvis: %d, ", m_see_status, nDltVisible);
                            // bol tu bug a neslo to => najjednoduchsia verzia
//                          bool bVisible = (((m_see_status&SEE_ANGLE) && (m_see_status&&SEE_DISTANCE)) ||
//                                         ((m_see_status&EXPECTED_ANGLE) && (m_see_status&EXPECTED_DISTANCE) && (nDltVisible<=2)));
                            bool bVisible = (nDltVisible<=2);
                            //! t0n0
							//fprintf(m_ff, "visible: %d\n", bVisible?1:0);
    
							return bVisible;
						}
	const CDiffuse2DVector &GetPosition() const
                        {
                            return m_position;
                        }
	const CDiffuse2DVector &GetPrevPosition() const
                        {
                            return m_prev_position;
                        }

	const CDiffuse2DVector &GetSpeed   () const 
                        {
                            return m_speed;
                        }
	const CDiffuse2DVector &GetAcceleration   () const
                        {
                            return m_acceleration;
                        }
	double              GetProbability()  const 
                        {
                            return m_probability;
                        }
	double              GetDirection() const 
                        {
                            return m_direction;
                        }
	void				SetPosition(const CDiffuse2DVector &p)
                        {
                            m_prev_position=m_position;
                            if((m_see_status & SEE_DISTANCE ||  m_see_status & EXPECTED_DISTANCE) ||
                               (m_see_status & SEE_ANGLE || m_see_status & EXPECTED_ANGLE))
                            {
                                m_see_status|=PREV_POSITION;
                            }
                            else
                            {
                                m_see_status&=!PREV_POSITION;
                            }
                            // FIX:
                            m_see_status|=SEE_DISTANCE;


                            m_position=p;
                        }
	void				SetSpeed   (const CDiffuse2DVector &s)
                        {
                            if((m_see_status & SEE_DISTANCE_CHANGE ||  m_see_status & EXPECTED_DISTANCE_CHANGE) ||
                               (m_see_status & SEE_ANGLE_CHANGE || m_see_status & EXPECTED_ANGLE_CHANGE))
                            {
                                m_see_status|=PREV_SPEED;
                            }
                            else
                            {
                                m_see_status&=!PREV_SPEED;
                            }
                            m_speed=s;
                        }
	void				SetAcceleration(const CDiffuse2DVector &a)
                        {
                            if(m_see_status & EXPECTED_ACCELERATION)
                            {
                                m_see_status|=PREV_ACCELERATION;
                            }
                            else
                            {
                                m_see_status&=!PREV_ACCELERATION;
                            }

                            m_acceleration=a;
                        }
	void				SetProbability(double p)
                        {
                            m_probability=p;
                        }
	void				SetDirection(double d)
                        {
                            m_direction=d;
                            // FIX:
                            m_see_status|=SEE_ANGLE;
                        }
//	void	virtual		Precompute(int stepcount,CGameCommand *command){}

	const CAngular2DVector &GetSeePosition() const
                        {
                            return m_see_position;
                        }
	const CAngular2DVector &GetSeeSpeed() const
                        {
                            return m_see_speed;
                        }
	int					 GetLastSeen() const
                        {
                            return m_last_seen;
                        }
	DWORD				 GetSeeStatus() const
                        {
                            return m_see_status;
                        }

	void SetSeePosition(const CAngular2DVector &pos)
                        {
                            m_see_position=pos;
                        }
	void SetSeeSpeed(const CAngular2DVector &speed)
                        {
                            m_see_speed=speed;
                        }
	void SetLastSeen(int ls)
                        {
                            m_last_seen=ls;
                        }
	void SetSeeStatus(DWORD stats,DWORD mask)
                        {
                            m_see_status=stats ^ (mask & (stats ^ m_see_status));
                        }

	void virtual Precompute();
};


class CWorldModel;



const int APP_GR_DEFAULT = 32;

class CAgent : public CMovingObject
{
protected:
	double			m_neck_direction, m_prev_neck_direction, m_angular_speed;
	double			m_stamina;
	double			m_effort;
	int				m_visual_settings;
	int				m_local_number;

	doublevector	m_approach_distances;

	int				m_number;
	ESide			m_side;
	double			m_goalie;

	CWorldModel		*m_my_world;

    CGameTime		m_time;
    
public:

    CAgent():CMovingObject()
			{
				m_visual_settings=0;
				SetApproachDistanceGranularity(APP_GR_DEFAULT);
                m_number=UNKNOWN;
                m_side=S_None;
                m_goalie=0;
				m_my_world=0;
			}

    CGameTime GetTime() const
    {
        return m_time;
    }

    void SetTime(CGameTime t)
    {
        m_time=t;
    }

	double	GetPrevNeckDirection() const
			{
				return m_prev_neck_direction;
			}
	double	GetAngularSpeed() const
			{
				return m_angular_speed;
			}
	double	GetNeckDirection() const
			{
				return m_neck_direction;
			}
	double	GetStamina() const
			{
				return m_stamina;
			}
	int		GetVisualSettings() const
			{
				return m_visual_settings;
			}
	int		GetLocalNumber() const
			{
				return m_local_number;
			}
    double  GetViewAngle() const
            {
                switch(m_visual_settings)
                {
                    case    VISUAL_RANGE_NARROW : return PI/2;
                    case    VISUAL_RANGE_NORMAL : return PI;
                    case    VISUAL_RANGE_WIDE   : return PI*2;
                }
                return 0;
            }
	int		GetApproachDistanceGranularity() const
			{
				return m_approach_distances.size();
			}
	double	GetApproachDistance(); const

	void	SetApproachDistanceGranularity(int gr)
			{
				m_approach_distances.resize(gr);
				for(int i=0;i<m_approach_distances.size();i++) m_approach_distances[i]=0;
			}
	void	SetAngularSpeed(double sp)
			{
				m_angular_speed=sp;
			}
	void	SetPrevNeckDirection(double nd)
			{
				m_prev_neck_direction=nd;
			}
	void	SetNeckDirection(double nd)
			{
				m_neck_direction=nd;
			}
	void	SetStamina(double st)
			{
				m_stamina=st;
			}
	void	SetVisualSettings(int vs)
			{
				m_visual_settings=vs;
			}
	void	SetLocalNumber(int ln)
			{	
				m_local_number=ln;
			}

//	bool command_used;


	int GetPlayerNumber() const
            {
                return m_number;
            }
	ESide GetPlayerSide() const
            {
                return m_side;
            }
	double GetGoalie() const
            {
                return m_goalie;
            }
	
	void SetPlayerNumber(int nr)
            {
                m_number=nr;
            }
	void SetPlayerSide(ESide ps)
            {
                m_side=ps;  
            }
	void SetGoalie(double gl)
            {
                m_goalie=gl;
            }

    CWorldModel *GetWorldModel() { return m_my_world; }

	double	virtual TestOwnership(const CSeenPlayer &player,const CAgent &local);//ughly&nasty
	void	virtual Update(CSeenPlayer &player,const CAgent &local);//ughly
	virtual CGameCommand *GetCommand();//nasty
	void	virtual	Precompute();//nasty

	void	UpdateApproachDistances(const CMovingObject &obj,const CAgent &local);//ughly
};


class CBall : public CMovingObject, public CDispatch
{
public:
	void UpdateBallPosition(CSeenObject _oBall,const CAgent &seeing);//ughly
	void virtual Precompute();//nasty

	CDispatch*	getDispatch( string sName );
	string		getAttribute( string sName );
};




typedef vector<CAgent*> Agents;


class CWorldModel
{
	CBall	*m_ball;

	Agents	m_identified_agents;
	Agents	m_unidentified_agents;
//	Agents	m_seen_agents;
public:    

	CWorldModel(const CWorldModel& cWorldModel);
	CWorldModel()
	{
		m_ball=new CBall;
	}
	void Recompute(CPerception *Perception,const CAgent &local);
	void Precompute();//nasty
	~CWorldModel()
	{
		delete m_ball;
		int i;
		for(i=0;i<m_identified_agents.size();i++)
		{
			delete m_identified_agents[i];
		}
		for(i=0;i<m_unidentified_agents.size();i++)
		{
			delete m_unidentified_agents[i];
		}

//		for(i=0;i<m_seen_agents.size();i++)
//		{
//			delete m_seen_agents[i];
//		}
	}

    CBall *GetBall() { return m_ball; };
    Agents *GetIdentifiedAgents() { return &m_identified_agents; }
    Agents *GetUnidentifiedAgents() { return &m_unidentified_agents; }

	int GetUnIdentifiedAgentsCount( void ) { return m_unidentified_agents.size(); };
	int GetIdentifiedAgentsCount( void ) { return m_identified_agents.size(); };

	CAgent* GetUnIdentifiedAgent( int index ) { return m_unidentified_agents[index]; };
	CAgent* GetIdentifiedAgent( int index ) { return m_identified_agents[index]; };
};


inline bool IsValid(CDiffuse2DVector local_pos,CDiffuse2DVector object_pos)
{
	double dist=(object_pos-local_pos).GetDistance ();
	double diff=object_pos.GetDiffusion ()-local_pos.GetDiffusion ();
	if(diff>0.1*dist+0.01)
	{
		return false;
	}
	return true;
}

#endif // _WORLD_OBJECTS_